"use strict";Object.defineProperty(exports, "__esModule", {value: true}); function _nullishCoalesce(lhs, rhsFn) { if (lhs != null) { return lhs; } else { return rhsFn(); } } function _optionalChain(ops) { let lastAccessLHS = undefined; let value = ops[0]; let i = 1; while (i < ops.length) { const op = ops[i]; const fn = ops[i + 1]; i += 2; if ((op === 'optionalAccess' || op === 'optionalCall') && value == null) { return undefined; } if (op === 'access' || op === 'optionalAccess') { lastAccessLHS = value; value = fn(value); } else if (op === 'call' || op === 'optionalCall') { value = fn((...args) => value.call(lastAccessLHS, ...args)); lastAccessLHS = undefined; } } return value; }// src/utils.ts
function debounce(func, {
  delay = 1500
} = {}) {
  let timer = void 0;
  return function(...args) {
    !timer && func.apply(this, args);
    timer = setTimeout(() => {
      timer = void 0;
    }, delay);
  };
}

// src/index.ts
function createPlainShiki(shiki) {
  const isSupported = () => "CSS" in globalThis && "highlights" in CSS;
  function mount(el, options) {
    const {
      lang,
      themes = {
        light: "min-light",
        dark: "min-dark"
      },
      selector = (theme) => `.${theme}`,
      watch = true,
      delay = 0
    } = options;
    const debouncedUpdate = delay > 0 ? debounce(update, { delay }) : update;
    const stylesheet = new CSSStyleSheet();
    document.adoptedStyleSheets.push(stylesheet);
    const loadLines = [];
    const names = /* @__PURE__ */ new Set();
    if (isSupported()) {
      watch && el.addEventListener("input", debouncedUpdate);
      update();
    }
    function dispose() {
      watch && el.removeEventListener("input", debouncedUpdate);
      const idx = document.adoptedStyleSheets.indexOf(stylesheet);
      document.adoptedStyleSheets.splice(idx, 1);
    }
    function resolveName(theme, color) {
      return `shiki-${theme}-${color.slice(1).toLowerCase()}`;
    }
    function patch(loads, oldLoads) {
      for (const { token, range } of oldLoads) {
        for (const theme in token.variants) {
          const { color } = token.variants[theme];
          const name = resolveName(theme, color);
          const highlight = CSS.highlights.get(name);
          if (!highlight) {
            continue;
          }
          highlight.delete(range);
        }
      }
      for (const { token, range } of loads) {
        for (const theme in token.variants) {
          const { color } = token.variants[theme];
          const name = resolveName(theme, color);
          const isDefault = theme === "light";
          let highlight = CSS.highlights.get(name);
          if (!highlight) {
            CSS.highlights.set(name, highlight = new Highlight());
            highlight.priority = isDefault ? 0 : 1;
          }
          if (!names.has(name)) {
            const rule = `${isDefault ? ":root" : selector(theme)}::highlight(${name}) { color: ${color}; }`;
            stylesheet.insertRule(rule);
            names.add(name);
          }
          highlight.add(range);
        }
      }
    }
    function diff(textLines) {
      let i = 0;
      for (; i < textLines.length; i++) {
        if (textLines[i] !== _optionalChain([loadLines, 'access', _ => _[i], 'optionalAccess', _2 => _2.text])) {
          return i;
        }
        for (const { range } of _nullishCoalesce(_optionalChain([loadLines, 'access', _3 => _3[i], 'optionalAccess', _4 => _4.loads]), () => ( []))) {
          if (range.collapsed) {
            return i;
          }
        }
      }
      return i;
    }
    function update() {
      const walker = document.createTreeWalker(el, NodeFilter.SHOW_TEXT);
      const textNodes = [];
      let node;
      while (node = walker.nextNode()) {
        textNodes.push(node);
      }
      const { innerText } = el;
      const textLines = innerText.split("\n");
      let i = diff(textLines);
      let lineOffset = textLines.slice(0, i).reduce((res, text) => res + text.length + 1, 0);
      let j = 0;
      let offset = 0;
      let isCorrect = false;
      findNodeAndOffset(lineOffset);
      for (; i < textLines.length; i++) {
        const text = textLines[i];
        const tokenLines = shiki.codeToTokensWithThemes(text, {
          lang,
          themes
        });
        const loads = [];
        for (const token of tokenLines[0]) {
          const [startNode, startOffset] = findNodeAndOffset(lineOffset + token.offset);
          const [endNode, endOffset] = findNodeAndOffset(lineOffset + token.offset + token.content.length);
          const range = document.createRange();
          range.setStart(startNode, startOffset);
          range.setEnd(endNode, endOffset);
          loads.push({ token, range });
        }
        const loadLine = loadLines[i] ??= {};
        loadLine.text = text;
        patch(loads, _nullishCoalesce(loadLine.loads, () => ( [])));
        loadLine.loads = loads;
        const lastGrammarState = shiki.getLastGrammarState(_nullishCoalesce(text, () => ( "")), { lang });
        if (loadLine.lastGrammarState !== lastGrammarState) {
          loadLine.lastGrammarState = lastGrammarState;
          lineOffset += text.length + 1;
        } else break;
      }
      function findNodeAndOffset(tokenOffset) {
        for (; j < textNodes.length; j++) {
          const node2 = textNodes[j];
          const { textContent } = node2;
          if (!textContent) {
            continue;
          }
          if (!isCorrect) {
            offset = innerText.indexOf(textContent, offset);
            isCorrect = true;
          }
          if (offset + textContent.length < tokenOffset) {
            offset += textContent.length;
            isCorrect = false;
          } else break;
        }
        return [textNodes[j], tokenOffset - offset];
      }
    }
    return {
      dispose,
      update
    };
  }
  return {
    get isSupported() {
      return isSupported();
    },
    mount
  };
}


exports.createPlainShiki = createPlainShiki;
